( cd usr/lib/news/bin ; rm -rf ovdb_upgrade )
( cd usr/lib/news/bin ; ln -sf ovdb_recover ovdb_upgrade )
( cd usr/bin ; rm -rf inews )
( cd usr/bin ; ln -sf /usr/lib/news/bin/inews inews )
( cd etc/rc.d ; rm -rf rc.news )
( cd etc/rc.d ; ln -sf /usr/lib/news/bin/rc.news rc.news )
